Imports RSToolKit.ReportService

Public Class frmProperties

    Private Enum ExportEnum
        XML
        CSV
        IMAGE
        PDF
        HTML4
        HTML3
        MHTML
        EXCEL
        HTMLOWC
    End Enum

    Private CI As CatalogItem
    Private RS As ReportServer

    Public WriteOnly Property ReportServer() As ReportServer
        Set(ByVal value As ReportServer)
            Me.RS = value
        End Set
    End Property


    Public WriteOnly Property CatalogItem() As CatalogItem
        Set(ByVal value As CatalogItem)
            Me.CI = value
        End Set
    End Property

    Private Sub frmEigenschaften_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        'TODO: Alle Properties des Originals merken und nur Updaten wenn sie auch wirklich verndert wurden
        'TODO: Beim Update sollten die vernderten Eigenschaften nicht einzeln geupdated werden sondern alle zusammen als array bergeben (ergibt dann nur eine Operation auf dem Server)
        UpdateLabels()

        Select Case CI.Type
            Case ItemTypeEnum.DataSource
                Me.TabControl1.TabPages.Remove(Me.TabControl1.TabPages.Item("tpSicherheit"))
                Me.TabControl1.TabPages.Remove(Me.TabControl1.TabPages.Item("tpSnapshots"))
                Me.TabControl1.TabPages.Remove(Me.TabControl1.TabPages.Item("tpDatasource"))
                Me.TabControl1.TabPages.Remove(Me.TabControl1.TabPages.Item("tpParameter"))
            Case ItemTypeEnum.Folder
                Me.TabControl1.TabPages.Remove(Me.TabControl1.TabPages.Item("tpSnapshots"))
                Me.TabControl1.TabPages.Remove(Me.TabControl1.TabPages.Item("tpDatasource"))
                Me.TabControl1.TabPages.Remove(Me.TabControl1.TabPages.Item("tpParameter"))
                Me.TabControl1.TabPages.Remove(Me.TabControl1.TabPages.Item("tpSicherheit"))
            Case ItemTypeEnum.LinkedReport

            Case ItemTypeEnum.Report
                FillSnapshots()
                FillSecurity()
                FillDataSource()
                FillParameter()
            Case ItemTypeEnum.Resource

            Case ItemTypeEnum.Unknown

        End Select
    End Sub

    Private Sub UpdateLabels()
        If (CI.Type = ItemTypeEnum.Report Or CI.Type = ItemTypeEnum.DataSource Or CI.Type = ItemTypeEnum.Folder) Then
            Me.txtCreatedBy.Text = IsNull(CI.CreatedBy)
            Me.txtCreatedAt.Text = IsNull(CI.CreationDate)
            Me.txtBeschreibung.Text = IsNull(CI.Description)
            If IsNull(CI.Hidden) = "True" Then
                Me.cbVersteckt.Checked = True
            Else
                Me.cbVersteckt.Checked = False
            End If
            Me.txtID.Text = "{" + IsNull(CI.ID) + "}"
            Me.txtModifiedBy.Text = IsNull(CI.ModifiedBy)
            Me.txtModifiedAt.Text = IsNull(CI.ModifiedDate)
            Me.txtName.Text = IsNull(CI.Name)
            Me.txtPath.Text = IsNull(CI.Path)
            tTip.SetToolTip(txtPath, txtPath.Text)
            If IsNumeric(IsNull(CI.Size)) Then
                Me.txtFilesize.Text = FormatNumber(CI.Size / 1024, 2).ToString + " KB"
            Else
                Me.txtFilesize.Text = IsNull(CI.Size)
            End If
            Me.txtDatatype.Text = IsNull(CI.Type)
        End If
    End Sub

    Private Sub FillSnapshots()
        If Me.CI.Type = ItemTypeEnum.Report Or Me.CI.Type = ItemTypeEnum.LinkedReport Then
            Me.lvSnapshots.Items.Clear()
            If Me.RS.ws.ListReportHistory(CI.Path).Length > 0 Then
                For Each x As RSToolKit.ReportService.ReportHistorySnapshot In Me.RS.ws.ListReportHistory(CI.Path)
                    Dim li As New ListViewItem
                    li.Text = x.CreationDate
                    li.Tag = x
                    li.SubItems.Add((CType(FormatNumber((x.Size / 1024), 2), String) + " KB"))
                    li.SubItems.Add(x.HistoryID)
                    li.ImageKey = "Snapshot"
                    Me.lvSnapshots.Items.Add(li)
                Next
            Else
                Me.lvSnapshots.Items.Add("No items available")
            End If
        Else
            Me.TabControl1.TabPages.Remove(Me.TabControl1.TabPages("tpSnapshots"))
        End If
    End Sub

    Private Sub FillSecurity()
        Me.lvSecurity.Items.Clear()
        If Me.CI.Type = ItemTypeEnum.Report Or Me.CI.Type = ItemTypeEnum.LinkedReport Then
            'Dim strPermissions() As String = Me.RS.ws.GetPermissions(CI.Path)
            Dim Policies() As RSToolKit.ReportService.Policy = Me.RS.ws.GetPolicies(CI.Path, True)
            For i As Integer = 0 To Policies.Length - 1
                Dim li As New ListViewItem
                li.Name = Policies(i).GroupUserName.ToString
                li.Text = Policies(i).GroupUserName.ToString
                li.ImageKey = "User"
                Me.lvSecurity.Items.Add(li)
            Next
        Else
            Me.lvSecurity.Items.Add("No items available")
        End If
    End Sub

    Private Sub FillDataSource()
        If Me.CI.Type = ItemTypeEnum.Report Or Me.CI.Type = ItemTypeEnum.LinkedReport Then
            Dim strPermissions() As String = Me.RS.ws.GetPermissions(CI.Path)
            Dim DataSource() As RSToolKit.ReportService.DataSource = Me.RS.ws.GetItemDataSources(CI.Path)
            For i As Integer = 0 To DataSource.Length - 1
                Dim li As New ListViewItem
                li.Name = DataSource(i).Name
                li.Text = DataSource(i).Name
                li.Tag = DataSource(i)
                li.ImageKey = "Database"
                Me.lvDataSource.Items.Add(li)
            Next
        Else
            Me.lvDataSource.Items.Add("No items available")
        End If
    End Sub

    Private Sub FillParameter()
        If Me.CI.Type = ItemTypeEnum.Report Or Me.CI.Type = ItemTypeEnum.LinkedReport Then
            Dim strPermissions() As String = Me.RS.ws.GetPermissions(CI.Path)
            Dim Parameter() As RSToolKit.ReportService.ReportParameter = Me.RS.ws.GetReportParameters(CI.Path, Nothing, False, Nothing, Nothing)
            For i As Integer = 0 To Parameter.Length - 1
                Dim li As New ListViewItem
                li.Name = Parameter(i).Name
                li.Text = Parameter(i).Name
                li.SubItems.Add(Parameter(i).Type.ToString)
                li.SubItems.Add(Parameter(i).Nullable.ToString)
                li.SubItems.Add(Parameter(i).State.ToString)
                li.ImageKey = "Parameter"
                Me.lvParameter.Items.Add(li)
            Next
        Else
            Me.lvParameter.Items.Add("No items available")
        End If
    End Sub

    Private Function IsNull(ByVal Value As Object) As String
        Dim ReturnValue As String
        If Not IsDBNull(Value) Then
            If Not IsNothing(Value) Then
                ReturnValue = Value.ToString
            Else
                ReturnValue = "n/a"
            End If
        Else
            ReturnValue = "n/a"
        End If
        Return ReturnValue
    End Function

    Private Sub btnOK_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnOK.Click
        If Me.UpdateItem Then
            Me.DialogResult = Windows.Forms.DialogResult.OK
        End If
    End Sub

    Private Sub btnCancel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnCancel.Click
        Me.DialogResult = Windows.Forms.DialogResult.Cancel
    End Sub

    Private Sub Button4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button4.Click
        If CI.Type = ItemTypeEnum.Report Or CI.Type = ItemTypeEnum.LinkedReport Then
            Try
                Me.RS.ws.CreateReportHistorySnapshot(CI.Path, Nothing)
                Me.FillSnapshots()
            Catch ex As Exception
                MsgBox("Ein Snapshot konnte nicht erstellt werden.", MsgBoxStyle.Critical, "Fehler")
                'MsgBox(ex.ToString)
            End Try
        End If
    End Sub

    Private Sub Button3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button3.Click
        If CI.Type = ItemTypeEnum.Report Or CI.Type = ItemTypeEnum.LinkedReport Then
            Try
                For Each n As ListViewItem In Me.lvSnapshots.SelectedItems
                    Dim x As RSToolKit.ReportService.ReportHistorySnapshot = n.Tag
                    Me.RS.ws.DeleteReportHistorySnapshot(CI.Path, x.HistoryID)
                Next
                Me.FillSnapshots()
            Catch ex As Exception
                If Me.lvSnapshots.SelectedItems.Count = 1 Then
                    MsgBox("Der Snapshot konnte nicht gelscht werden.", MsgBoxStyle.Critical, "Fehler")
                Else
                    MsgBox("Die Snapshots konnten nicht gelscht werden.", MsgBoxStyle.Critical, "Fehler")
                End If
            End Try
        End If
    End Sub

    Private Sub lvAdobePDF_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lvAdobePDF.Click
        Export(ExportEnum.PDF)
    End Sub

    Private Sub lvExcel_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lvExcel.Click
        Export(ExportEnum.EXCEL)
    End Sub

    Private Sub lvTIFF_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lvTIFF.Click
        Export(ExportEnum.IMAGE)
    End Sub

    Private Sub lvHTML3_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lvHTML3.Click
        Export(ExportEnum.HTML3)
    End Sub

    Private Sub lvHTML4_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lvHTML4.Click
        Export(ExportEnum.HTML4)
    End Sub

    Private Sub lvCSV_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lvCSV.Click
        Export(ExportEnum.CSV)
    End Sub

    Private Sub lvXML_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lvXML.Click
        Export(ExportEnum.XML)
    End Sub

    Private Sub lvWebArchiv_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lvWebArchiv.Click
        Export(ExportEnum.MHTML)
    End Sub

    Private Sub lvHTMLOWC_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles lvHTMLOWC.Click
        Export(ExportEnum.HTMLOWC)
    End Sub

    Private Sub Export(ByVal ExportFormat As ExportEnum)
        If CI.Type = ItemTypeEnum.Report Or CI.Type = ItemTypeEnum.LinkedReport Then
            Try
                For Each n As ListViewItem In Me.lvSnapshots.SelectedItems
                    Dim x As RSToolKit.ReportService.ReportHistorySnapshot = n.Tag
                    Render(ExportFormat, x.HistoryID)
                Next
            Catch ex As Exception
                MsgBox("Der Snapshot konnte nicht exportiert werden.", MsgBoxStyle.Critical, "Fehler")
            End Try
        End If
    End Sub

    Private Sub Render(ByVal ExportFormat As ExportEnum, ByVal HistoryID As String)
        Try
            Dim strFormat As String = Nothing
            Select Case ExportFormat
                Case ExportEnum.CSV
                    Me.sfExport.Filter = "CSV-Datei (*.csv)|.csv"
                    strFormat = "CSV"
                Case ExportEnum.EXCEL
                    Me.sfExport.Filter = "Microsoft Excel (*.xls)|.xls"
                    strFormat = "EXCEL"
                Case ExportEnum.HTMLOWC
                    Me.sfExport.Filter = "HTML Office Web Components (*.html)|.html"
                    strFormat = "HTMLOWC"
                Case ExportEnum.PDF
                    Me.sfExport.Filter = "Adobe Acrobat (*.pdf)|.pdf"
                    strFormat = "PDF"
                Case ExportEnum.XML
                    Me.sfExport.Filter = "XML-Datei (*.xml)|.xml"
                    strFormat = "XML"
                Case ExportEnum.IMAGE
                    Me.sfExport.Filter = "Image (*.tiff)|.tiff"
                    strFormat = "IMAGE"
                Case ExportEnum.MHTML
                    Me.sfExport.Filter = "Web-Archiv (*.mhtml)|.mhtml"
                    strFormat = "MHTML"
                Case ExportEnum.HTML3
                    Me.sfExport.Filter = "HTML 3.2 (*.html)|.html"
                    strFormat = "HTML3.2"
                Case ExportEnum.HTML4
                    Me.sfExport.Filter = "HTML 4.0 (*.html)|.html"
                    strFormat = "HTML4.0"
            End Select
            Me.sfExport.FileName = ""
            If Me.sfExport.ShowDialog = Windows.Forms.DialogResult.OK Then
                Dim results() As Byte = Nothing
                'results = Me.RS.ws.Render(CI.Path, strFormat, HistoryID, "", Nothing, Nothing, "", "", "", Nothing, Nothing, Nothing)
                Dim stream As System.IO.FileStream = System.IO.File.OpenWrite(Me.sfExport.FileName)
                stream.Write(results, 0, results.Length)
                stream.Close()
            End If
        Catch ex As Exception
            MsgBox(ex.ToString)
        End Try
    End Sub

    Private Sub LschenToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles LschenToolStripMenuItem.Click
        If CI.Type = ItemTypeEnum.Report Or CI.Type = ItemTypeEnum.LinkedReport Then
            Try
                For Each n As ListViewItem In Me.lvSnapshots.SelectedItems
                    Dim x As RSToolKit.ReportService.ReportHistorySnapshot = n.Tag
                    Me.RS.ws.DeleteReportHistorySnapshot(CI.Path, x.HistoryID)
                Next
                Me.FillSnapshots()
            Catch ex As Exception
                MsgBox("The snapshot could not be deleted.", MsgBoxStyle.Critical, "Error")
            End Try
        End If
    End Sub

    Private Sub btnUpdate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs)
        UpdateItem()
    End Sub

    Private Function UpdateItem() As Boolean
        Dim bRet As Boolean = False
        Dim props(0) As [Property]

        Dim itemPath As String = CI.Path
        Try
            Dim setProp As New [Property]()

            'Description
            If Not Me.txtBeschreibung.Text = "n/a" Then 'Nur updaten wenn auch eine angegeben wurde
                setProp.Name = "Description"
                setProp.Value = Me.txtBeschreibung.Text
                props(0) = setProp
                RS.ws.SetProperties(itemPath, props)
            End If

            'Hidden
            setProp = New [Property]()
            setProp.Name = "Hidden"
            setProp.Value = Me.cbVersteckt.Checked
            props(0) = setProp
            RS.ws.SetProperties(itemPath, props)

            bRet = True
        Catch ex As Exception
            MsgBox(ex.ToString)
            MsgBox("Ihre nderungen konnten nicht bernommen werden.")
            bRet = False
        End Try
        Return bRet
    End Function

    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        Dim frmp As New frmPolicies
        frmp.ReportServer = Me.RS
        frmp.Item = Me.CI
        If frmp.ShowDialog = Windows.Forms.DialogResult.OK Then
            Me.FillSecurity()
        End If
    End Sub

    Private Sub Button1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button1.Click
        Try
            Dim Pol() As RSToolKit.ReportService.Policy
            Pol = Me.RS.ws.GetPolicies(CI.Path, True)
            For Each llv As ListViewItem In Me.lvSecurity.SelectedItems
                For i As Integer = 0 To Pol.Length - 1
                    If Not IsNothing(Pol(i)) Then
                        If Pol(i).GroupUserName = llv.Text Then
                            Array.Clear(Pol, i, 1)
                            Exit For
                        End If
                    End If
                Next
            Next
            Dim j As Integer = 0
            For i As Integer = 0 To Pol.Length - 1
                If Not IsNothing(Pol(i)) Then j += 1
            Next
            Dim Policy(j - 1) As RSToolKit.ReportService.Policy
            j = 0
            For i As Integer = 0 To Pol.Length - 1
                If Not IsNothing(Pol(i)) Then
                    Policy(j) = Pol(i)
                    j += 1
                End If
            Next
            RS.ws.SetPolicies(CI.Path, Policy)
            Me.FillSecurity()
        Catch ex As Exception
            MsgBox(ex.ToString)
        End Try
    End Sub

    Private Sub btnTest_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnTest.Click
        MsgBox("Not implemented...", MsgBoxStyle.Information, "RSToolKit BETA Version")
    End Sub

End Class